/* EntryFileDialog.java
 *
 * created: Mon Dec  7 1998
 *
 * This file is part of Artemis
 *
 * Copyright (C) 1998-2003  Genome Research Limited
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/pathsoft/repository/powmap/diana/components/EntryFileDialog.java,v 1.36 2003/11/14 17:34:23 kmr Exp $
 */

package diana.components;

import diana.Options;
import uk.ac.sanger.pathogens.*;
import uk.ac.sanger.pathogens.embl.Entry;
import uk.ac.sanger.pathogens.embl.DocumentEntryFactory;
import uk.ac.sanger.pathogens.embl.ReadFormatException;
import uk.ac.sanger.pathogens.embl.EntryInformation;
import uk.ac.sanger.pathogens.embl.EntryInformationException;
import uk.ac.sanger.pathogens.embl.InvalidRelationException;
import uk.ac.sanger.pathogens.embl.InvalidKeyException;
import uk.ac.sanger.pathogens.embl.InvalidQualifierException;
import uk.ac.sanger.pathogens.embl.SimpleEntryInformation;
import uk.ac.sanger.pathogens.embl.Key;
import uk.ac.sanger.pathogens.embl.Qualifier;
import uk.ac.sanger.pathogens.embl.QualifierInfo;
import uk.ac.sanger.pathogens.embl.QualifierInfoException;
import uk.ac.sanger.pathogens.embl.ReadEvent;
import uk.ac.sanger.pathogens.embl.ReadListener;

import java.awt.*;
import java.io.*;

/**
 *  This class is a FileDialog that can read EMBL Entry objects.
 *
 *  @author Kim Rutherford
 *  @version $Id: EntryFileDialog.java,v 1.36 2003/11/14 17:34:23 kmr Exp $
 **/

public class EntryFileDialog extends FileDialog {
  /**
   *  Create a new EntryFileDialog component.
   *  @param owner The component where this dialog was created.
   **/
  public EntryFileDialog (final Frame owner) {
    super (owner, "Select a file...", FileDialog.LOAD);
    this.owner = owner;
  }

  /**
   *  Used to remember the directory the FileDialog was in when the user
   *  pressed OK.  This is used as the starting directory next time.
   **/
  private static String last_directory = null;

  /**
   *  Return an uk.ac.sanger.pathogens.embl.Entry object representing the file
   *  the user has selected with the dialog or null if the read failed for any
   *  reason.
   *  @param entry_information The EntryInformation to use when reading.  This
   *    supplies the list of valid keys and qualifiers.  If a key or qualifier
   *    is read that is incompatible with this EntryInformation object the
   *    EntryInformation will be changed to cope.
   *  @param listener The object to which InputStreamProgressEvents will be
   *    send while reading.
   *  @exception EntryInformationException Thrown if an Entry using the given
   *    EntryInformation object cannot contain the Key, Qualifier or
   *    Key/Qualifier combination of one of the features in the Document.
   *  @param show_progress If true a InputStreamProgressDialog will be shown
   *    while loading.
   **/
  public Entry getEntry (final EntryInformation entry_information,
                         final InputStreamProgressListener listener,
                         final boolean show_progress) {
    if (last_directory == null) {
      if (Options.getOptions ().getProperty ("default_directory") != null) {
        final String default_directory =
          Options.getOptions().getProperty("default_directory");
        setDirectory(default_directory);
      }
    } else {
      setDirectory (last_directory);
    }

    show ();

    if (getDirectory () == null || getFile () == null) {
      return null;
    }

    last_directory = getDirectory ();

    final File file = new File (getDirectory (), getFile ());

    return getEntryFromFile (owner, file, entry_information, listener,
                             show_progress);
  }

  /**
   *  This exists only to get around a bug in the 1.1/1.2 code generator,
   *  which generates unverifiable code.
   *  @param frame Used when creating MessageDialog components.
   *  @param entry_file The file to read the Entry from.
   *  @param file_document The Document to read the Entry from (should be made
   *    from entry_file).
   *  @param entry_information The EntryInformation to use when reading.  This
   *    supplies the list of valid keys and qualifiers.  If a key or qualifier
   *    is read that is incompatible with this EntryInformation object the
   *    EntryInformation will be changed to cope.
   *  @param errors One String is added to this for each error.
   **/
  private static Entry
    getEntryFromFileHelper (final Frame frame,
                            final File entry_file,
                            final FileDocument file_document,
                            final EntryInformation entry_information)
      throws ReadFormatException, IOException {

    final DialogReadListener read_event_logger =
      new DialogReadListener (frame, entry_file);

    final Entry new_entry;

    try {
      if (entry_file != null) {
        new_entry = DocumentEntryFactory.makeDocumentEntry (entry_information,
                                                            file_document,
                                                            read_event_logger);

      } else {
        new MessageDialog (frame,
                           "creating new entry: " + entry_file.getName ());

        new_entry = DocumentEntryFactory.makeDocumentEntry (file_document,
                                                            read_event_logger);
      }
    } catch (EntryInformationException e) {
      throw new Error ("internal error - unexpected exception: " + e);
    }

    if (read_event_logger.seenMessage ()) {
      final YesNoDialog yes_no_dialog =
        new YesNoDialog (frame,
                         "there were warnings while reading - view now?");

      if (yes_no_dialog.getResult ()) {
        Splash.showLog ();
      }
    }

    return new_entry;
  }

  /**
   *  Read and return an Entry from the given File.
   *  @param frame Used when creating MessageDialog components.
   *  @param entry_file The file to read the Entry from.
   *  @param entry_information The EntryInformation to use when reading.  This
   *    supplies the list of valid keys and qualifiers.  If a key or qualifier
   *    is read that is incompatible with this EntryInformation object the
   *    EntryInformation will be changed to cope.
   *  @param listener The object to which InputStreamProgressEvents will be
   *    send while reading.
   *  @param show_progress If true a InputStreamProgressDialog will be shown
   *    while loading.
   **/
  public static Entry
    getEntryFromFile (final Frame frame,
                      final File entry_file,
                      final EntryInformation entry_information,
                      final InputStreamProgressListener listener,
                      final boolean show_progress) {
    final InputStreamProgressDialog progress_dialog;

    if (show_progress) {
      progress_dialog =
        new InputStreamProgressDialog (frame, "Reading ...",
                                       "Reading from " +
                                       entry_file.getName (), false);
    } else {
      progress_dialog = null;
    }

    try {
      final InputStreamProgressListenerVector listeners =
        new InputStreamProgressListenerVector ();

      if (listener != null) {
        listeners.add (listener);
      }

      if (progress_dialog != null) {
        listeners.add (progress_dialog.getInputStreamProgressListener ());
      }

      final FileDocument file_document =
        new FileProgressDocument (entry_file, listeners);

      return getEntryFromFileHelper (frame, entry_file, file_document,
                                     entry_information);
    } catch (ReadFormatException e) {
      final String message =
        "failed to open " + entry_file.getName () + ": " + e.getMessage () +
        (e.getLineNumber () > 1 ?
         " at line: " + e.getLineNumber () :
         "");
      System.out.println (message);
      new MessageDialog (frame, message);
    } catch (FileNotFoundException e) {
      final String message =
        "failed to open " + entry_file.getName () + ": file not found";
      new MessageDialog (frame, message);
    } catch (IOException e) {
      final String message =
        "failed to open " + entry_file.getName () + ": " + e.getMessage ();
      new MessageDialog (frame, message);
    } finally {
      if (progress_dialog != null) {
        progress_dialog.dispose ();
      }
    }

    return null;
  }

  /**
   *  Implements ReadListener by logging all ReadEvents.
   **/
  private static class DialogReadListener implements ReadListener {
    /**
     *  Create a new DialogReadListener.  Use frame to create Dialogs.
     *  @param file The file we are reading from.
     **/
    public DialogReadListener (final Frame frame, final File file) {
      this.frame = frame;
      this.file = file;
    }

    /**
     *  Implementation of ReadListener.
     **/
    public void notify (final ReadEvent event) {
      if (file == null) {
        Splash.getLogger ().log (event.getMessage () + "\n");
      } else {
        Splash.getLogger ().log ("while reading from " + file + ": " +
                                 event.getMessage () + "\n");
      }
      seen_message = true;
    }

    /**
     *  Return true if and only if notify() has been called.
     **/
    public boolean seenMessage () {
      return seen_message;
    }

    private Frame frame;
    private File file;

    private boolean seen_message = false;
  }

  /**
   *  The Frame reference that was passed to the constructor.
   **/
  private Frame owner = null;
}

